@@ -5,7 +5,7 @@ dataprep.eda
 import tempfile
 
 from bokeh.io import output_file, output_notebook
-from .basic import compute, plot, render
+from .distribution import compute, plot, render
 from .correlation import compute_correlation, plot_correlation, render_correlation
 from .missing import compute_missing, plot_missing, render_missing
 from .create_report import create_report
